from typing import List, Optional

from pydantic import Field, BaseModel

class BotConfig(BaseModel):
    """
    Kaiheila Bot 配置类
    :配置项:
      - ``token``: Kaiheila 开发者中心获得
    """

    token: str

    class Config:
        extra = "ignore"
        allow_population_by_field_name = True


class Config(BaseModel):
    """
    Kaiheila 配置类

    :配置项:

      - ``kaiheila_bots`` : Kaiheila 开发者中心获得
      - ``compress`` : 是否开启压缩, 默认为 False

    :示例:

    .. code-block:: default
    
        bots = [{"token": "token1"}, {"token": "token2"}]
    """
    kaiheila_bots: List["BotConfig"] = Field(default_factory=list)
    compress: Optional[bool] = Field(default=False)

    class Config:
        extra = "allow"
        allow_population_by_field_name = True



import json
from dataclasses import dataclass
from typing import Any, Type, Tuple, Union, Mapping, Iterable, Dict, cast, Optional

from deprecated import deprecated
from nonebot.adapters import Message as BaseMessage
from nonebot.adapters import MessageSegment as BaseMessageSegment
from nonebot.typing import overrides

from .exception import UnsupportedMessageType, UnsupportedMessageOperation
from .utils import unescape_kmarkdown, escape_kmarkdown

msg_type_map = {
    "text": 1,
    "image": 2,
    "video": 3,
    "file": 4,
    "audio": 8,
    "kmarkdown": 9,
    "card": 10,
}

rev_msg_type_map = {}
for msg_type, code in msg_type_map.items():
    rev_msg_type_map[code] = msg_type

# 根据协议消息段类型显示消息段内容
segment_text = {
    "text": "[文字]",
    "image": "[图片]",
    "video": "[视频]",
    "file": "[文件]",
    "audio": "[音频]",
    "kmarkdown": "[KMarkdown消息]",
    "card": "[卡片消息]",
}


class MessageSegment(BaseMessageSegment["Message"]):
    """
    开黑啦 协议 MessageSegment 适配。具体方法参考协议消息段类型或源码。

    https://developer.kaiheila.cn/doc/event/message
    """

    # 已知：
    # command/shell_command使用message_seg = Message[0]; str(message_seg) if message_seg.is_text()
    # startswith/endswith/keyword使用event.get_plaintext()
    # regex使用str(Message)

    @classmethod
    @overrides(BaseMessageSegment)
    def get_message_class(cls) -> Type["Message"]:
        return Message

    def __str__(self) -> str:
        if self.type in ["text", "kmarkdown"]:
            return str(self.data["content"])
        elif self.type == "at":
            return str(f"@{self.data['user_name']}")
        else:
            return segment_text.get(self.type, "[未知类型消息]")

    @property
    def plain_text(self):
        if self.type == "text":
            return self.data["content"]
        elif self.type == "kmarkdown":
            return self.data["raw_content"]
        else:
            return ""

    @overrides(BaseMessageSegment)
    def __add__(self, other: Union[str, "MessageSegment", Iterable["MessageSegment"]]) -> "Message":
        return Message(self.conduct(other))

    @overrides(BaseMessageSegment)
    def __radd__(self, other: Union[str, "MessageSegment", Iterable["MessageSegment"]]) -> "Message":
        if isinstance(other, str):
            other = MessageSegment(self.type, {"content": other})
        return Message(other.conduct(self))

    def conduct(self, other: Union[str, "MessageSegment", Iterable["MessageSegment"]]) -> "MessageSegment":
        """
        连接两个或多个 MessageSegment，必须为纯文本段或 KMarkdown 段
        """

        if isinstance(other, str) or isinstance(other, MessageSegment):
            other = [other]
        msg = Message([self, *other])
        msg.reduce()

        if len(msg) != 1:
            raise UnsupportedMessageOperation("必须为纯文本段或 KMarkdown 段")
        else:
            return msg[0]

    @overrides(BaseMessageSegment)
    def is_text(self) -> bool:
        if self.type == "kmarkdown":
            return self.data["raw_content"] == self.data["content"]
        else:
            return self.type == "text"

    @staticmethod
    @deprecated("用 KMarkdown 语法 (met)用户id/here/all(met) 代替")
    def at(user_id: str) -> "MessageSegment":
        return MessageSegment.KMarkdown(f"(met){user_id}(met)", user_id)

    @staticmethod
    def text(text: str) -> "MessageSegment":
        return MessageSegment("text", {"content": text})

    @staticmethod
    def image(file_key: str) -> "MessageSegment":
        return MessageSegment("image", {"file_key": file_key})

    @staticmethod
    def video(file_key: str,
              title: Optional[str] = None) -> "MessageSegment":
        return MessageSegment("video", {
            "file_key": file_key,
            "title": title,
        })

    @staticmethod
    def file(file_key: str,
             title: Optional[str] = None) -> "MessageSegment":
        return MessageSegment("file", {
            "file_key": file_key,
            "title": title,
        })

    @staticmethod
    def audio(file_key: str,
              title: Optional[str] = None,
              cover_file_key: Optional[str] = None) -> "MessageSegment":
        return MessageSegment("audio", {
            "file_key": file_key,
            "title": title,
            "cover_file_key": cover_file_key
        })

    @staticmethod
    def KMarkdown(content: str, raw_content: Optional[str] = None) -> "MessageSegment":
        """
        构造KMarkdown消息段

        @param content: KMarkdown消息内容（语法参考：https://developer.kookapp.cn/doc/kmarkdown）
        @param raw_content: （可选）消息段的纯文本内容
        """
        if raw_content is None:
            raw_content = ""

        return MessageSegment("kmarkdown", {
            "content": content,
            "raw_content": raw_content
        })

    @staticmethod
    def Card(content: Any) -> "MessageSegment":
        """
        构造卡片消息

        @param content: KMarkdown消息内容（语法参考：https://developer.kookapp.cn/doc/cardmessage）
        """
        if not isinstance(content, str):
            content = json.dumps(content)

        return MessageSegment("card", {
            "content": content
        })

    @staticmethod
    def quote(msg_id: str) -> "MessageSegment":
        return MessageSegment("quote", {
            "msg_id": msg_id
        })


class Message(BaseMessage[MessageSegment]):
    """
    开黑啦 v3 协议 Message 适配。
    """

    @classmethod
    @overrides(BaseMessage)
    def get_segment_class(cls) -> Type[MessageSegment]:
        return MessageSegment

    @staticmethod
    @overrides(BaseMessage)
    def _construct(
            msg: Union[str, Mapping, Iterable[Mapping]]
    ) -> Iterable[MessageSegment]:
        if isinstance(msg, Mapping):
            msg = cast(Mapping[str, Any], msg)
            yield MessageSegment(msg["type"], msg.get("content") or {})
        elif isinstance(msg, Iterable) and not isinstance(msg, str):
            for seg in msg:
                yield MessageSegment(seg["type"], seg.get("content") or {})
        elif isinstance(msg, str):
            yield MessageSegment("text", {"content": msg})

    @overrides(BaseMessage)
    def extract_plain_text(self) -> str:
        return "".join(seg.plain_text for seg in self)

    def reduce(self) -> None:
        """合并消息内连续的纯文本段和 KMarkdown 段。"""
        index = 1
        while index < len(self):
            prev = self[index - 1]
            cur = self[index]
            if prev.type == "text" and cur.type == "text":
                self[index - 1] = MessageSegment(prev.type, {
                    "content": prev.data["content"] + cur.data["content"]
                })
                del self[index]
            elif prev.type == "kmarkdown" and cur.type == "kmarkdown":
                self[index - 1] = MessageSegment(prev.type, {
                    "content": prev.data["content"] + cur.data["content"],
                    "raw_content": prev.data["raw_content"] + cur.data["raw_content"],
                })
                del self[index]
            elif prev.type == "kmarkdown" and cur.type == "text":
                self[index - 1] = MessageSegment(prev.type, {
                    "content": prev.data["content"] + escape_kmarkdown(cur.data["content"]),
                    "raw_content": prev.data["raw_content"] + cur.data["content"],
                })
                del self[index]
            elif prev.type == "text" and cur.type == "kmarkdown":
                self[index - 1] = MessageSegment(prev.type, {
                    "content": escape_kmarkdown(prev.data["content"]) + cur.data["content"],
                    "raw_content": prev.data["content"] + cur.data["raw_content"],
                })
                del self[index]
            else:
                index += 1


def _convert_to_card_message(msg: Message) -> MessageSegment:
    cards = []
    modules = []

    for seg in msg:
        if seg.type == 'card':
            if len(modules) != 0:
                cards.append({"type": "card", "theme": "none", "size": "lg", "modules": modules})
                modules = []
            cards.extend(json.loads(seg.data['content']))
        elif seg.type == "text":
            modules.append(
                {
                    "type": "section",
                    "text": {"type": "plain-text", "content": seg.data["content"]},
                }
            )
        elif seg.type == "kmarkdown":
            modules.append(
                {
                    "type": "section",
                    "text": {"type": "kmarkdown", "content": seg.data["content"]},
                }
            )
        elif seg.type == "image":
            modules.append(
                {
                    "type": "container",
                    "elements": [{"type": "image", "src": seg.data["file_key"]}],
                }
            )
        elif seg.type in ("audio", "video", "file"):
            mod = {
                "type": seg.type,
                "src": seg.data["file_key"],
            }
            if seg.data.get("title") is not None:
                mod["title"] = seg.data["title"]
            if seg.data.get("cover_file_key") is not None:
                mod["cover"] = seg.data["cover_file_key"]
            modules.append(mod)
        else:
            raise UnsupportedMessageType(seg.type)

    if len(modules) != 0:
        cards.append({"type": "card", "theme": "none", "size": "lg", "modules": modules})

    return MessageSegment.Card(cards)


@dataclass
class MessageSerializer:
    """
    开黑啦 协议 Message 序列化器。
    """
    message: Message

    def serialize(self, for_send: bool = True) -> Tuple[int, str]:
        if len(self.message) != 1:
            self.message = self.message.copy()
            self.message.reduce()

            if len(self.message) != 1:
                # 转化为卡片消息发送
                return MessageSerializer(Message(_convert_to_card_message(self.message))).serialize()

        msg_type = self.message[0].type
        msg_type_code = msg_type_map[msg_type]
        # bot 发送消息只支持"text", "kmarkdown", "card"
        # 经测试还支持"image", "video", "file"
        if msg_type in ("text", "kmarkdown", "card"):
            return msg_type_code, self.message[0].data['content']
        elif msg_type in ("image", "video", "file"):
            return msg_type_code, self.message[0].data['file_key']
        elif msg_type == "audio":
            if not for_send:
                return msg_type_code, self.message[0].data['file_key']
            else:
                # 转化为卡片消息发送
                return MessageSerializer(Message(_convert_to_card_message(self.message))).serialize()
        else:
            raise UnsupportedMessageType(msg_type)


@dataclass
class MessageDeserializer:
    """
    开黑啦 协议 Message 反序列化器。
    """
    type_code: int
    data: Dict

    def __post_init__(self):
        self.type = rev_msg_type_map.get(self.type_code, "")

    def deserialize(self) -> Message:
        if self.type == "text":
            return Message(MessageSegment.text(self.data["content"]))
        elif self.type == "image":
            return Message(MessageSegment.image(self.data["content"]))
        elif self.type == "video":
            return Message(MessageSegment.video(self.data["attachments"]["url"]))
        elif self.type == "file":
            return Message(MessageSegment.file(self.data["attachments"]["url"]))
        elif self.type == "kmarkdown":
            content = self.data["content"]
            raw_content = self.data["kmarkdown"]["raw_content"]

            # raw_content默认strip掉首尾空格，但是开黑啦本体的聊天界面中不会strip，所以这里还原了
            unescaped = unescape_kmarkdown(content)
            is_plain_text = unescaped.strip() == raw_content
            if is_plain_text:
                raw_content = unescaped

            # 如果是KMarkdown消息是纯文本，直接构造纯文本消息
            # 目的是让on_command等依赖__str__的规则能够在消息存在转义字符时正常工作
            if is_plain_text:
                return Message(MessageSegment.text(raw_content))
            else:
                return Message(MessageSegment.KMarkdown(content, raw_content))
        elif self.type == "card":
            return Message(MessageSegment.Card(self.data["content"]))
        else:
            return Message(MessageSegment(self.type, self.data))



from .model import *


class ApiClient:

    async def asset_create(self, *, file) -> URL:
        ...

    async def blacklist_create(
            self, *,
            guild_id: str,
            target_id: str,
            remark: Optional[str] = ...,
            del_msg_days: Optional[str] = ...
    ) -> None:
        ...

    async def blacklist_delete(
            self, *,
            guild_id: str,
            target_id: str
    ) -> None:
        ...

    async def blacklist_list(self, *, guild_id: str) -> BlackListsReturn:
        ...

    async def channelRole_create(
            self, *,
            channel_id: str,
            type: Optional[str] = ...,
            value: Optional[str] = ...
    ) -> ChannelRoleReturn:
        ...

    async def channelRole_delete(
            self, *,
            channel_id: str,
            type: Optional[str] = ...,
            value: Optional[str] = ...,
    ) -> None:
        ...

    async def channelRole_index(self, *, channel_id: str) -> ChannelRoleInfo:
        """获取频道角色权限详情

        Args:
            channel_id (str): 频道ID

        Returns:
            ChannelRoleInfo: 频道角色权限详情
        """
        ...

    async def channelRole_update(
            self, *,
            channel_id: str,
            type: Optional[str] = ...,
            value: Optional[str] = ...,
            allow: Optional[int] = ...,
            deny: Optional[int] = ...
    ) -> ChannelRoleReturn:
        ...

    async def channel_create(
            self, *,
            guild_id: str,
            name: str,
            parent_id: Optional[str] = ...,
            type: Optional[int] = ...,
            limit_amount: Optional[int] = ...,
            voice_quality: Optional[str] = ...,
            is_category: Optional[int] = ...
    ) -> Channel:
        ...

    async def channel_delete(self, *, channel_id: str) -> None:
        ...

    async def channel_update(
            self, *,
            channel_id: str,
            name: Optional[str] = ...,
            topic: Optional[str] = ...,
            slow_mode: Optional[int] = ...
    ) -> Channel:
        ...

    async def channel_list(
            self, *,
            guild_id: str,
            type: Optional[int] = ...,
            page: Optional[int] = ...,
            page_size: Optional[int] = ...
    ) -> ChannelsReturn:
        ...

    async def channel_moveUser(
            self, *,
            target_id: str,
            user_ids: List[int],
    ) -> None:
        ...

    async def channel_userList(self, *, channel_id: str) -> List[User]:
        ...

    async def channel_view(self, *, target_id: str) -> Channel:
        ...

    async def directMessage_addReaction(
            self, *,
            msg_id: str,
            emoji: str
    ) -> None:
        ...

    async def directMessage_create(
            self, *,
            content: str,
            type: Optional[int] = ...,
            target_id: Optional[str] = ...,
            chat_code: Optional[str] = ...,
            quote: Optional[str] = ...,
            nonce: Optional[str] = ...
    ) -> MessageCreateReturn:
        ...

    async def directMessage_delete(self, *, msg_id: str) -> None:
        """删除私信聊天消息

        Args:
            msg_id (str): 消息 id
        """
        ...

    async def directMessage_deleteReaction(
            self, *,
            msg_id: str,
            emoji: str,
            user_id: Optional[str] = ...
    ) -> None:
        ...

    async def directMessage_list(
            self, *,
            chat_code: Optional[str] = ...,
            target_id: Optional[str] = ...,
            msg_id: Optional[str] = ...,
            flag: Optional[str] = ...,
            page: Optional[int] = ...,
            page_size: Optional[int] = ...
    ) -> DirectMessagesReturn:
        """获取私信聊天消息列表

        Args:
            chat_code (str, optional):
                私信会话 Code,chat_code与target_id必须传一个.
            target_id (str, optional):
                目标用户 id,后端会自动创建会话. 有此参数之后可不传chat_code参数.
            msg_id (str, optional):
                参考消息 id,不传则查询最新消息.
            flag (str, optional):
                查询模式,有三种模式可以选择. 不传则默认查询最新的消息.
            page (int, optional): 目标页数.
            page_size (int, optional): 当前分页消息数量, 默认 `50`.

        Returns:
            DirectMessagesReturn：获取私信聊天消息列表返回信息
        """
        ...

    async def directMessage_reactionList(
            self, *,
            msg_id: str,
            emoji: str
    ) -> List[ReactionUser]:
        ...

    async def directMessage_update(
            self, *,
            content: str,
            msg_id: Optional[str] = ...,
            quote: Optional[str] = ...
    ) -> None:
        """更新私信聊天消息

        Args:
            content (str):
                消息 id
            msg_id (str, optional):
                消息内容
            quote (str, optional):
                回复某条消息的msgId. 如果为空,则代表删除回复,不传则无影响.
        """
        ...

    async def gateway_index(self, *, compress: Optional[int] = ...) -> URL:
        ...

    async def guildEmoji_create(
            self, *,
            guild_id: str,
            emoji: Optional[bytes] = ...,
            name: Optional[str] = ...
    ) -> GuildEmoji:
        ...

    async def guildEmoji_delete(self, *, id: str) -> None:
        ...

    async def guildEmoji_list(
            self, *,
            guild_id: str,
            page: Optional[int] = ...,
            page_size: Optional[int] = ...
    ) -> GuildEmojisReturn:
        ...

    async def guildEmoji_update(
            self, *,
            id: str,
            name: str
    ) -> None:
        ...

    async def guildMute_create(
            self, *,
            guild_id: str = ...,
            target_id: str = ...,
            type: int = ...
    ) -> None:
        ...

    async def guildMute_delete(
            self, *,
            guild_id: str = ...,
            target_id: str = ...,
            type: int = ...
    ) -> None:
        ...

    async def guildMute_list(
            self, *,
            guild_id: str,
            return_type: Optional[str] = ...
    ) -> None:
        ...

    async def guildRole_create(
            self, *,
            guild_id: str,
            name: Optional[str] = ...
    ) -> Role:
        ...

    async def guildRole_delete(
            self, *,
            guild_id: str,
            role_id: int
    ) -> None:
        ...

    async def guildRole_grant(
            self, *,
            guild_id: str,
            user_id: str,
            role_id: int
    ) -> GuilRoleReturn:
        ...

    async def guildRole_list(
            self, *,
            guild_id: str,
            page: Optional[int] = ...,
            page_size: Optional[int] = ...
    ) -> RolesReturn:
        ...

    async def guildRole_revoke(
            self, *,
            guild_id: str,
            user_id: str,
            role_id: int
    ) -> GuilRoleReturn:
        ...

    async def guildRole_update(
            self, *,
            guild_id: str,
            role_id: int,
            name: Optional[str] = ...,
            color: Optional[int] = ...,
            hoist: Optional[int] = ...,
            mentionable: Optional[int] = ...,
            permissions: Optional[int] = ...
    ) -> Role:
        ...

    async def guild_kickout(self, *, guild_id: str, target_id: str) -> None:
        ...

    async def guild_leave(self, *, guild_id: str) -> None:
        ...

    async def guild_list(
            self, *,
            page: Optional[int] = ...,
            page_size: Optional[int] = ...,
            sort: Optional[str] = ...
    ) -> GuildsReturn:
        """获取当前用户加入的服务器列表

        Args:
            page (Optional[int], optional): 目标页数
            page_size (Optional[int], optional): 每页数据数量
            sort (Optional[str], optional): 代表排序的字段

        Returns:
            GuildsReturn: 当前用户加入的服务器列表返回信息
        """
        ...

    async def guild_nickname(
            self, *,
            guild_id: str = ...,
            nickname: Optional[str] = ...,
            user_id: Optional[str] = ...
    ) -> None:
        ...

    async def guild_userList(
            self, *,
            guild_id: str,
            channel_id: Optional[str] = ...,
            search: Optional[str] = ...,
            role_id: Optional[int] = ...,
            mobile_verified: Optional[int] = ...,
            active_time: Optional[int] = ...,
            joined_at: Optional[int] = ...,
            page: Optional[int] = ...,
            page_size: Optional[int] = ...,
            filter_user_id: Optional[str] = ...
    ) -> GuildUsersRetrun:
        """获取服务器中的用户列表

        Args:
            guild_id (str): 服务器 id
            channel_id (Optional[str], optional): 频道 id
            search (Optional[str], optional): 搜索关键字，在用户名或昵称中搜索
            role_id (Optional[int], optional): 角色 ID，获取特定角色的用户列表
            mobile_verified (Optional[int], optional): 只能为0或1，0是未认证，1是已认证
            active_time (Optional[int], optional): 根据活跃时间排序，0是顺序排列，1是倒序排列
            joined_at (Optional[int], optional): 根据加入时间排序，0是顺序排列，1是倒序排列
            page (Optional[int], optional): 目标页数
            page_size (Optional[int], optional): 每页数据数量
            filter_user_id (Optional[str], optional): 获取指定 id 所属用户的信息
        Returns:
            GuildsReturn: 服务器中的用户列表返回信息
        """
        ...

    async def guild_view(self, *, guild_id: str) -> Guild:
        """获取服务器详情

        Args:
            guild_id (str): 服务器id

        Returns:
            Guild: 服务器详情
        """
        ...

    async def intimacy_index(self, *, user_id: str) -> IntimacyIndexReturn:
        ...

    async def intimacy_update(
            self, *,
            user_id: str,
            score: Optional[int] = ...,
            social_info: Optional[str] = ...,
            img_id: Optional[str] = ...
    ) -> None:
        ...

    async def invite_create(
            self, *,
            guild_id: Optional[str] = ...,
            channel_id: Optional[str] = ...,
            duration: Optional[int] = ...,
            setting_times: Optional[int] = ...,
    ) -> URL:
        ...

    async def invite_delete(
            self, *,
            url_code: str,
            guild_id: Optional[str] = ...,
            channel_id: Optional[str] = ...
    ) -> None:
        ...

    async def invite_list(
            self, *,
            guild_id: Optional[str] = ...,
            channel_id: Optional[str] = ...,
            page: Optional[int] = ...,
            page_size: Optional[int] = ...
    ) -> InvitesReturn:
        ...

    async def message_addReaction(
            self, *,
            msg_id: str,
            emoji: str
    ) -> None:
        ...

    async def message_create(
            self, *,
            content: str,
            target_id: str,
            type: Optional[int] = ...,
            quote: Optional[str] = ...,
            nonce: Optional[str] = ...,
            temp_target_id: Optional[str] = ...
    ) -> MessageCreateReturn:
        ...

    async def message_delete(self, *, msg_id: str) -> None:
        ...

    async def message_deleteReaction(
            self, *,
            msg_id: str,
            emoji: str,
            user_id: Optional[str] = ...
    ) -> None:
        ...

    async def message_list(
            self, *,
            target_id: str,
            msg_id: Optional[str] = ...,
            pin: Optional[int] = ...,
            flag: Optional[str] = ...,
            page_size: Optional[int] = ...
    ) -> ChannelMessagesReturn:
        ...

    async def message_reactionList(
            self, *,
            msg_id: str,
            emoji: str
    ) -> List[ReactionUser]:
        ...

    async def message_update(
            self, *,
            msg_id: str,
            content: str,
            quote: Optional[str] = ...,
            temp_target_id: Optional[str] = ...
    ) -> None:
        ...

    async def message_view(self, *, msg_id: str) -> ChannelMessage:
        ...

    async def userChat_create(self, *, target_id: str) -> UserChat:
        ...

    async def userChat_delete(self, *, chat_code: str) -> None:
        ...

    async def userChat_list(
            self, *,
            page: Optional[int] = ...,
            page_size: Optional[int] = ...
    ) -> UserChatsReturn:
        ...

    async def userChat_view(self, *, chat_code: str) -> UserChat:
        ...

    async def user_me(self) -> User:
        ...

    async def user_offline(self) -> None:
        """下线机器人"""
        ...

    async def user_view(
            self, *,
            user_id: str,
            guild_id: Optional[str] = ...
    ) -> User:
        ...



import re
from io import BytesIO, BufferedReader
from os import PathLike
from pathlib import Path
from typing import Any, Union, TYPE_CHECKING, BinaryIO, Dict, Optional, Literal, Callable

from nonebot.adapters import Bot as BaseBot
from nonebot.message import handle_event
from nonebot.typing import overrides

from .api import ApiClient
from .event import Event, MessageEvent
from .message import Message, MessageSegment, MessageSerializer
from .utils import log

if TYPE_CHECKING:
    from .event import Event
    from .adapter import Adapter
    from .message import Message, MessageSegment


def _check_at_me(bot: "Bot", event: MessageEvent):
    """
    :说明:

      检查消息开头或结尾是否存在 @机器人，去除并赋值 ``event.to_me``

    :参数:

      * ``bot: Bot``: Bot 对象
      * ``event: Event``: Event 对象
    """
    if not isinstance(event, MessageEvent):
        return

    # ensure message not empty
    # if not event.message:
    #     event.message.append(MessageSegment.text(""))

    if event.message_type == "private":
        event.to_me = True
    else:
        if event.message[0].type == "kmarkdown" and bot.self_id in event.extra.mention:
            event.to_me = True
            met_str = f"(met){bot.self_id}(met)"
            raw_met_str = f"@{bot.self_name}"

            content: str = event.message[0].data["content"].strip()
            raw_content: str = event.message[0].data["raw_content"].strip()
            if content.startswith(met_str):
                content = content[len(met_str):].lstrip()
                raw_content = raw_content[len(raw_met_str):].lstrip()
            elif content.endswith(met_str):
                content = content[:-len(met_str)].rstrip()
                raw_content = raw_content[:-len(raw_met_str)].rstrip()
            event.message[0].data["content"] = content
            event.message[0].data["raw_content"] = raw_content
            event.message[0].data["is_plain_text"] = True


def _check_nickname(bot: "Bot", event: MessageEvent):
    """
    :说明:

      检查消息开头是否存在昵称，去除并赋值 ``event.to_me``

    :参数:

      * ``bot: Bot``: Bot 对象
      * ``event: Event``: Event 对象
    """
    first_msg_seg = event.message[0]
    if first_msg_seg.type != "text":
        return

    first_text = first_msg_seg.data["content"]

    nicknames = set(filter(lambda n: n, bot.config.nickname))
    if nicknames:
        # check if the user is calling me with my nickname
        nickname_regex = "|".join(nicknames)
        m = re.search(rf"^({nickname_regex})([\s,，]*|$)", first_text, re.IGNORECASE)
        if m:
            nickname = m.group(1)
            log("DEBUG", f"User is calling me {nickname}")
            event.to_me = True
            first_msg_seg.data["content"] = first_text[m.end():]


async def send(
        bot: "Bot",
        event: Event,
        message: Union[str, Message, MessageSegment],
        reply_sender: bool = False,
        is_temp_msg: bool = False,
        **kwargs: Any,
) -> Any:
    # 构造参数
    params = {**kwargs}

    # quote
    if reply_sender:
        params.setdefault("quote", getattr(event, "message_id"))

    # message_type
    if event.channel_type == 'GROUP':
        params.setdefault("message_type", "channel")

        # temp_target_id
        if is_temp_msg:
            params.setdefault("user_id", getattr(event, "user_id"))

        # target_id
        if getattr(event, "target_id", None):
            params.setdefault("channel_id", getattr(event, "target_id"))
    else:
        params.setdefault("message_type", "private")

        # target_id
        if getattr(event, "target_id", None):
            params.setdefault("user_id", getattr(event, "user_id"))

    params.setdefault("message", message)

    return await bot.send_msg(**params)


class Bot(BaseBot, ApiClient):
    """
    Kaiheila Bot 适配。
    """

    send_handler: Callable[
        ["Bot", Event, Union[str, Message, MessageSegment], bool, bool], Any
    ] = send

    def __init__(self, adapter: "Adapter", self_id: str, name: str, token: str):
        """
        :参数:

          * ``adapter: Adapter``: 适配器
          * ``self_id: str``: 机器人 ID
          * ``name: str``: 机器人用户名
          * ``token``: 机器人 token

        """
        super().__init__(adapter, self_id)
        self.self_name: str = name
        self.token: str = token

    @overrides(BaseBot)
    async def call_api(self, api: str, **data) -> Any:
        """
        :说明:

          调用 kaiheila 协议 API

        :参数:

          * ``api: str``: API 名称
          * ``**data: Any``: API 参数

        :返回:

          - ``Any``: API 调用返回数据

        :异常:

          - ``NetworkError``: 网络错误
          - ``ActionFailed``: API 调用失败
        """
        return await super().call_api(api, **data)

    async def handle_event(self, event: Event) -> None:
        if isinstance(event, MessageEvent):
            _check_at_me(self, event)
            _check_nickname(self, event)
            pass

        await handle_event(self, event)

    @overrides(BaseBot)
    async def send(
            self,
            event: Event,
            message: Union[str, Message, MessageSegment],
            reply_sender: bool = False,
            is_temp_msg: bool = False,
            **kwargs,
    ) -> Any:
        """
        :说明:

          根据 ``event``  向触发事件的主体发送消息。

        :参数:

          * ``event: Event``: Event 对象
          * ``message: Union[str, Message, MessageSegment]``: 要发送的消息
          * ``reply_sender: bool``: 是否回复原消息
          * ``is_temp_msg: bool``: 是否临时消息
          * ``**kwargs``: 覆盖默认参数

        :返回:

          - ``Any``: API 调用返回数据

        :异常:

          - ``ValueError``: 缺少 ``user_id``, ``channel_id``
          - ``NetworkError``: 网络错误
          - ``ActionFailed``: API 调用失败
        """
        return await self.__class__.send_handler(self, event, message, reply_sender, is_temp_msg, **kwargs)

    async def send_private_msg(
            self,
            *,
            user_id: str,
            message: Union[str, Message, MessageSegment],
            quote: Optional[str] = None
    ) -> Dict[str, Any]:
        """发送私聊消息。

            user_id: 对方用户ID
            message: 要发送的内容，字符串类型将作为纯文本消息发送
            quote: 回复某条消息的消息ID
        """
        return await self.send_msg(message_type="private",
                                   user_id=user_id,
                                   message=message,
                                   quote=quote)

    async def send_channel_msg(
            self,
            *,
            channel_id: str,
            message: Union[str, Message, MessageSegment],
            quote: Optional[str] = None
    ) -> Dict[str, Any]:
        """发送频道消息。

            channel_id: 频道ID
            message: 要发送的内容，字符串类型将作为纯文本消息发送
            quote: 回复某条消息的消息ID
        """
        return await self.send_msg(message_type="channel",
                                   channel_id=channel_id,
                                   message=message,
                                   quote=quote)

    async def send_temp_msg(
            self,
            *,
            user_id: str,
            channel_id: str,
            message: Union[str, Message, MessageSegment],
            quote: Optional[str] = None
    ) -> Dict[str, Any]:
        """发送频道临时消息。该消息不会存数据库，但是会在频道内只给该用户推送临时消息。用于在频道内针对用户的操作进行单独的回应通知等。

            channel_id: 频道ID
            message: 要发送的内容，字符串类型将作为纯文本消息发送
            quote: 回复某条消息的消息ID
        """
        return await self.send_msg(message_type="temp",
                                   user_id=user_id,
                                   channel_id=channel_id,
                                   message=message,
                                   quote=quote)

    async def send_msg(
            self,
            *,
            message_type: Literal['private', 'channel', 'temp', ''] = '',
            user_id: Optional[str] = None,
            channel_id: Optional[str] = None,
            message: Union[str, Message, MessageSegment],
            quote: Optional[str] = None
    ) -> Dict[str, Any]:
        """发送消息。

        参数:
            message_type: 消息类型，支持 `private`、`channel`、`temp`，分别对应私聊、频道，如不传入，则根据传入的 `*_id` 参数判断
            user_id: 对方用户ID（消息类型为 `private`、`temp` 时需要）
            channel_id: 频道ID（消息类型为 `channel`、`temp` 时需要）
            message: 要发送的内容，字符串类型将作为纯文本消息发送
            quote: 回复某条消息的消息ID
        """
        # 接口文档：
        # https://developer.kaiheila.cn/doc/http/direct-message#%E5%8F%91%E9%80%81%E7%A7%81%E4%BF%A1%E8%81%8A%E5%A4%A9%E6%B6%88%E6%81%AF
        # https://developer.kaiheila.cn/doc/http/message#%E5%8F%91%E9%80%81%E9%A2%91%E9%81%93%E8%81%8A%E5%A4%A9%E6%B6%88%E6%81%AF
        params = {}

        # type & content
        if isinstance(message, Message):
            new_message = Message()
            # 提取message中的quote消息段
            for seg in message:
                if seg.type == 'quote':
                    if not quote:
                        quote = seg.data["msg_id"]
                else:
                    new_message.append(seg)

            params["type"], params["content"] = MessageSerializer(new_message).serialize()
        elif isinstance(message, MessageSegment):
            params["type"], params["content"] = MessageSerializer(Message(message)).serialize()
        else:
            params["type"], params["content"] = 1, message

        # quote
        if quote:
            params["quote"] = quote

        # target_id & api
        if message_type == "channel":
            params["target_id"] = channel_id
            api = "message/create"
        elif message_type == "private":
            params["target_id"] = user_id
            api = "direct-message/create"
        elif message_type == "temp":
            params["target_id"] = channel_id
            params["temp_target_id"] = user_id
            api = "message/create"
        else:
            if channel_id and not user_id:
                params["target_id"] = channel_id
                api = "message/create"
            elif not channel_id and user_id:
                params["target_id"] = user_id
                api = "direct-message/create"
            elif channel_id and user_id:
                params["target_id"] = channel_id
                params["temp_target_id"] = user_id
                api = "message/create"
            else:
                raise ValueError(f"channel_id 和 user_id 不能同时为 None")

        return await self.call_api(api, **params)

    async def upload_file(self, file: Union[str, PathLike[str], BinaryIO, bytes],
                          filename: Optional[str] = None) -> str:
        """
        上传文件。

        参数:
            file: 文件，可以是文件路径（str, PathLike[str]）、打开的文件流（BinaryIO）、或二进制数据（bytes）
            filename: 文件名

        返回值:
            文件的 URL
        """
        if isinstance(file, BytesIO):
            file = file.getvalue()
        elif isinstance(file, BufferedReader):
            file = file.read()
        elif isinstance(file, str) or isinstance(file, Path):
            with open(file, "rb") as img:
                file = img.read()
        # 经过测试，服务器会用从文件读取到的mime覆盖掉我们传过去的mime
        file = (filename or "upload-file", file, "application/octet-stream")
        result = await self.asset_create(file=file)
        return result.url
